@@ -1,9 +1,9 @@
 """
-Modules borrowed from https://github.com/kennethzhao24/mobilevit_pytorch
+Modules adapted/borrowed from https://github.com/kennethzhao24/mobilevit_pytorch
 for easier loading of official weights.
 Official code and weights: https://github.com/apple/ml-cvnets
 
-Adapted by Sovit Ranjan Rath.
+Put together by Sovit Ranjan Rath.
 """
 
 import torch.nn as nn
@@ -16,11 +16,69 @@ from .layers.conv_bn_act import ConvBlock
 from .layers.norm import get_normalization_layer
 from .layers.weight_init import initialize_weights
 from .layers.global_pool import GlobalPool
-from .layers.attention import Attention
 from .layers.activation import get_activation_fn
 
 from argparse import Namespace
 
+class MHSA(nn.Module):
+    """
+        Multi-head self attention: https://arxiv.org/pdf/1706.03762
+    """
+    def __init__(self, 
+                 embed_dim, 
+                 num_heads, 
+                 attn_dropout = 0.0, 
+                 bias = True
+                 ):
+        """
+            :param embed_dim: embedding dimension
+            :param num_heads: number of attention heads
+            :param attn_dropout: attention dropout
+            :param bias: use bias or not
+        """
+        super(MHSA, self).__init__()
+        assert embed_dim % num_heads == 0, "Got: embed_dim={} and num_heads={}".format(embed_dim, num_heads)
+
+        self.qkv_proj = nn.Linear(in_features=embed_dim, out_features=3*embed_dim, bias=bias)
+
+        self.attn_dropout = nn.Dropout(p=attn_dropout)
+        self.out_proj = nn.Linear(in_features=embed_dim, out_features=embed_dim, bias=bias)
+
+        self.head_dim = embed_dim // num_heads
+        self.num_heads = num_heads
+        self.scaling = self.head_dim ** -0.5
+        self.softmax = nn.Softmax(dim=-1)
+
+    def forward(self, x):
+        # [B x N x C]
+        b_sz, n_patches, _ = x.shape
+
+        # linear projection to qkv
+        # [B x N x C] --> [B x N x 3 x h x C]
+        qkv = (self.qkv_proj(x).reshape(b_sz, n_patches, 3, self.num_heads, -1))
+        # [B x N x 3 x h x C] --> [B x h x 3 x N x C]
+        qkv = qkv.transpose(1, 3)
+        # [B x h x 3 x N x C] --> [B x h x N x C] x 3
+        q, k, v = qkv[:, :, 0], qkv[:, :, 1], qkv[:, :, 2]
+        q = q * self.scaling
+        # [B x h x N x C] --> [B x h x c x N]
+        k = k.transpose(2, 3)
+
+        # compute attention score
+        # [B x h x N x c] x [B x h x c x N] --> [B x h x N x N]
+        attn = torch.matmul(q, k)
+        attn = self.softmax(attn)
+        attn = self.attn_dropout(attn)
+
+        # weighted sum
+        # [B x h x N x N] x [B x h x N x c] --> [B x h x N x c]
+        out = torch.matmul(attn, v)
+        # [B x h x N x c] --> [B x N x h x c] --> [B x N x C=ch]
+        out = out.transpose(1, 2).reshape(b_sz, n_patches, -1)
+        out = self.out_proj(out)
+
+        return out
+
 class TransformerEncoder(nn.Module):
     """
         Transfomer Encoder
@@ -49,11 +107,11 @@ class TransformerEncoder(nn.Module):
 
         self.pre_norm_mha = nn.Sequential(
             get_normalization_layer(opts=opts, norm_type=transformer_norm_layer, num_features=embed_dim),
-            Attention(
+            MHSA(
                 embed_dim, 
                 num_heads, 
-                dim_head=embed_dim//num_heads,
-                dropout=attn_dropout
+                # dim_head=embed_dim//num_heads,
+                attn_dropout=attn_dropout
             ),
             nn.Dropout(p=dropout)
         )
@@ -663,7 +721,7 @@ def mobilevit_s(num_classes=1000, pretrained=False):
         ckpt = torch.hub.load_state_dict_from_url(
             'https://docs-assets.developer.apple.com/ml-research/models/cvnets/classification/mobilevit_s.pt'
         )
-        model.load_state_dict(ckpt, strict=False)
+        model.load_state_dict(ckpt)
 
     in_features = model.classifier.fc.in_features
     model.classifier.fc = nn.Linear(
@@ -679,7 +737,7 @@ def mobilevit_xs(num_classes=1000, pretrained=False):
         ckpt = torch.hub.load_state_dict_from_url(
             'https://docs-assets.developer.apple.com/ml-research/models/cvnets/classification/mobilevit_xs.pt'
         )
-        model.load_state_dict(ckpt, strict=False)
+        model.load_state_dict(ckpt)
     in_features = model.classifier.fc.in_features
     model.classifier.fc = nn.Linear(
         in_features=in_features, out_features=num_classes, bias=True
@@ -694,7 +752,7 @@ def mobilevit_mini(num_classes=1000, pretrained=False):
         ckpt = torch.hub.load_state_dict_from_url(
             'https://github.com/kennethzhao24/mobilevit_pytorch/blob/main/weights/mobilevit_mini.pt'
         )
-        model.load_state_dict(ckpt, strict=False)
+        model.load_state_dict(ckpt)
     in_features = model.classifier.fc.in_features
     model.classifier.fc = nn.Linear(
         in_features=in_features, out_features=num_classes, bias=True
@@ -709,7 +767,7 @@ def mobilevit_xxs(num_classes=1000, pretrained=False):
         ckpt = torch.hub.load_state_dict_from_url(
             'https://docs-assets.developer.apple.com/ml-research/models/cvnets/classification/mobilevit_xxs.pt'
         )
-        model.load_state_dict(ckpt, strict=False)
+        model.load_state_dict(ckpt)
     in_features = model.classifier.fc.in_features
     model.classifier.fc = nn.Linear(
         in_features=in_features, out_features=num_classes, bias=True
